/*
   Code to generate reports based on templates and ITEM_LIST data structures
*/
/*
 * For licensing terms, see the file COPYING
 * For contact information, see the file AUTHORS
 *
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "stuff.h"
#include "item.h"
#include "report.h"
#include "stream.h"

static char* replace_str(char *base,char **strs);
const static char * tokens[] = {"%channel%", "%item%","%link%", "%description%", "%date%", "%score%"};
const static size_t token_lens[] = {9, 6, 6, 13, 6, 7};

#define outw(x) stream_write (stream, (x), strlen ((x)))
void generate_report(ITEM_LIST *il,FN_OPTIONS *fn)
{
	char * template;
	char ** parts;
	FILE * out;
	STREAM * stream;
	char * r;
	int i;
	ITEM *t;
	char *prev_channel = NULL;
	template = read_file (fn->template_file);
	if (template == NULL)
	{
		printf("Unable to open template file\n");
		return;
	}
	
	stream = stream_create (1024);
	
	parts=split_str(template,"%%",5);
	/*ASSERT(parts[4]!=NULL);
	ASSERT(parts[3]!=NULL);
	ASSERT(parts[2]!=NULL);
	ASSERT(parts[1]!=NULL);
	ASSERT(parts[0]!=NULL);*/
	if (parts[0]==NULL ||
			parts[1]==NULL||
			parts[2]==NULL||
			parts[3]==NULL||
			parts[4]==NULL)
	{
		printf("Invalid template file\n");
		return;
	}

	/*Generate the header section*/
	outw(parts[0]);
	
	for(i=0;i<il->count;i++)
	{
		char score[10];
		char *strs[]={NULL,NULL,NULL,NULL,NULL,NULL};
	    t=il->items[i];
		strs[0]=t->channel;
		strs[1]=t->item;
		strs[2]=t->link;
		strs[3]=t->description;
		strs[4]="";
		sprintf(score,"%i",t->score);
		strs[5]=score;
		
		
		/* Check if we have a new channel. If so, generate the channel
		 * section */
		if (prev_channel!=NULL && t->channel!=NULL &&
				(strcmp(prev_channel,t->channel)!=0))
		{
 			r=replace_str(parts[1],strs);
			outw (r);
 			fn_free(r);
		}
		
		else if (prev_channel==NULL && t->channel!=NULL)
		{
			r=replace_str(parts[1],strs);
			outw (r);
			fn_free(r);
		}

		
		prev_channel=t->channel;
		
		if (t->item!=NULL && t->link!=NULL)
		{
			r=replace_str(parts[2],strs);
			outw (r);
			fn_free(r);
		}
		else
		{
			if (t->item!=NULL)
			{
				//outw(stream,"<BR>%s<BR>\n",t->item);
				r=replace_str(parts[2],strs);
				outw (r);
				fn_free(r);
			}
		}
		if (t->description!=NULL)
		{
			//outw(stream,"%s<BR>\n",t->description);
			r=replace_str(parts[3],strs);
			outw (r);
			fn_free(r);
		}
	}
	outw (parts[4]);
	if (has_option(OPT_DUMP_REPORT))
		out = stdout;
	else
	{
		out=fopen(fn->report_file,"w");
		if (!out)
		{
			printf("Unable to open output file\n");
			stream_free (stream);
			return;
		}
	}

	fwrite (stream->buffer, 1, stream->size, out);
	
	if (has_option(OPT_DUMP_REPORT))
	fclose (out);

	stream_free (stream);

	for(i = 0;i<5;i++)
	{
		//printf("%i\n",i);
		//printf(parts[i]);
		fn_free(parts[i]);
	}
}
char **split_str(char * base,char *token,unsigned int elements)
{
	int i;
	size_t pos;
	size_t tok_len;
	char *crap;
	char *b;
	char **c;
	c=(char **)fn_malloc(elements*sizeof(char *));
	tok_len=strlen(token);
	for(i=0;i<elements;i++)
	{
		c[i]=NULL;
	}
	i=0;
	pos=0;
	b=crap=base;
	while (i<elements)
	{
		crap=strstr(base,token);
		if (crap==NULL)break;
		pos=(unsigned int)crap-(unsigned int)base;
		c[i]=(char *)fn_malloc(pos+1);
		c[i][pos]=0;
		memcpy(c[i],base,pos);
		i++;
		base+=pos+tok_len;
		
	}
	if (i<elements)
	{
		pos=(unsigned int)(b)+strlen(b)-(size_t)base;
		c[i]=(char *)fn_malloc(pos+1);
		memcpy(c[i],base,pos);
		c[i][pos]=0;
	}
	return c;
}
#define check_token(i) if (strncmp(haystack,tokens[i],token_lens[i])==0)return i
static int token_index(char *haystack)
{
	if (*haystack!='%')
		return -1;
	switch (*(haystack+1))
	{
		case 'c':check_token(0);return -1;
		case 'i':check_token(1);return -1;
		case 'l':check_token(2);return -1;
		case 'd':check_token(3);
				 check_token(4);return -1;
		case 's':check_token(5);return -1;
		default: return -1;
	}
}

static char * replace_str(char *base,char **strs)
{
	char *new_str;
	int i,res;
	size_t base_len;
	int length=0;
	base_len=strlen(base);
	i=0;
	while (i<base_len)
	{
		if ( (res=token_index(base+i))>=0)
		{
			length+=strlen(strs[res]);
			i+=token_lens[res];
		}
		else
		{
			length++;
			i++;
		}
	}
	length++;
	new_str=(char *)fn_malloc(sizeof(char)*(length+6));//+5
	new_str[length]=0;
	i=0;
	while (i<length)
	{
		if ( (res=token_index(base))>=0)
		{
			memcpy(new_str+i,strs[res],strlen(strs[res]));
			i+=strlen(strs[res]);
			base+=token_lens[res];
		}
		else
		{
			*(new_str+i++)=*(base++);
		}
	}
	
	return new_str;
}
//#define SCORE_FIELD(field,str,_score) if (strstr(t->field,str))score+=_scorey
#define SCORE_FIELD(field) if (t->field && strstr(t->field,s->pattern))score+=s->score
int compare(const void *_first,const void *_second)
{
	ITEM *first,*second;
	int i;
	first=*(ITEM **)_first;
	second=*(ITEM **)_second;
	i=first->score-second->score;
	if (i>0)return -1;
	if (i<0)return 1;
	return 0;
}
void score_items(ITEM_LIST *il,SCORE_LIST *sl)
{
	int i,score,j;
	ITEM *t;
	SCORE_ITEM *s;
	for (i=0;i<il->count;++i)
	{
		score=0;
		t=il->items[i];
// 		SCORE_FIELD(item,"Linux",100);
		for(j=0;j<sl->count;j++)
		{
			s=sl->items[j];
			switch(s->category)
			{
				case CAT_TITLE:		  SCORE_FIELD(item);break;
				case CAT_DESCRIPTION: SCORE_FIELD(description);break;
				case CAT_CHANNEL:	  SCORE_FIELD(channel);break;
				case CAT_LINK:		  SCORE_FIELD(link);break;
				default:
			}
			
		}
		t->score=score;
	}
 	qsort(il->items,il->count,sizeof(ITEM *),compare);
}
/*main()
{
	char **d;
	int i;
	char *s="this\nis a \ttemplate\n%%\npart2\nofit\n%%\npart3\n\n%%\npart4";
	d=split_str(s,"\n%%\n",10);
	for (i=0;i<10;++i)
		if (d[i])printf("%s\n---8<---\n",d[i]);
	
}*/
